import os
from rdflib import Graph, URIRef
from rest_framework.generics import ListCreateAPIView, RetrieveUpdateDestroyAPIView, ListAPIView
from rest_framework import permissions

from django.http import HttpResponse, JsonResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.parsers import JSONParser

from reqman.apps.reqtool.models.requirements import Prefix, Boilerplate, BoilerplateGroup, PREFIX_CHOICES, LOGIC_CONNECTIVE
from reqman.apps.reqtool.models.requirements import BoilerplateData, InferenceResults, BoilerplateGroupClassesInstances

from reqman.apps.reqtool.rest_api.serializers.requirements import BoilerplateSerializer, PrefixSerializer, BoilerplateDataSerializer, BoilerplateGroupSerializer, InferSerializer, BoilerplateGroupClassesInstancesSerializer
from reqman.apps.permissions import IsOwnerOrReadOnly

from reqman.apps.reqtool.models.main_req import Main
from reqman.apps.reqtool.models.suffix_req import Suffix
from reqman.apps.reqtool.rest_api.services import fuseki, inference

from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.exceptions import APIException

from django.shortcuts import get_object_or_404
import requests, json
from requests.auth import HTTPBasicAuth
from reqman.settings.environment import env



class OntologiesChoicesViewSet(APIView):
    '''Get current Ontologies from Fuseki server'''

    def get(self, request):
        data={'datasets':''}
        try : 
            url = requests.get(env("FUSEKI_URL")+"/$/datasets", auth = HTTPBasicAuth(env("FUSEKI_Username"), env("FUSEKI_Password")))
            text = url.text
            data = json.loads(text)
        except:
            pass
        list_ontology=['/Default']
        for i in data['datasets'] :
            list_ontology.append(i['ds.name'])
        ONTOLOGIES_CHOICES = tuple((str(n), str(n)) for n in (list_ontology))
        response = Response(ONTOLOGIES_CHOICES)
        return response


class BoilerplateGroupCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = BoilerplateGroupSerializer


    def get_queryset(self):
        queryset_boil = BoilerplateGroup.objects.filter(boilerplate_owner = self.request.user)
        return queryset_boil

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save(boilerplate_owner=self.request.user)

class BoilerplateGroupList(ListAPIView):
    """
    API view to retrieve list of posts based on title of Boilerplate
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = BoilerplateGroupSerializer
    queryset = BoilerplateGroup.objects.all()
    

class BoilerplateGroupDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = BoilerplateGroupSerializer
    queryset = BoilerplateGroup.objects.all()

    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.save()

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        try:
            os.system('rm -rf ../../Ontologies/' + instance.title_bgroup +'/')
        except:
            print('No file exist')
        instance.delete()


class BoilerplateListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = BoilerplateSerializer

    def get_queryset(self):
        group_gb = self.kwargs['groupboil']
        queryset_boil = Boilerplate.objects.filter(owner = self.request.user, group_of_boilerplate = group_gb)
        return queryset_boil

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save(owner=self.request.user)


class BoilerplateList(ListAPIView):
    """
    API view to retrieve list of posts based on title of Boilerplate
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = BoilerplateSerializer


    def get_queryset(self):
        group_gb = self.kwargs['groupboil']
        title = self.kwargs['title_1']
        queryset_title = Boilerplate.objects.filter(title = title, group_of_boilerplate = group_gb)
        return queryset_title

   

class BoilerplateDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly,
                      IsOwnerOrReadOnly]
    serializer_class = BoilerplateSerializer
    queryset = Boilerplate.objects.all()

    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.validated_data
        bd_title = instance['title']
        BoilerplateData.objects.filter(boilerplate_data_id=self.kwargs['pk'], group_of_boilerplate_data = instance['group_of_boilerplate']).update(title_data = bd_title)
        instance = serializer.save()
        

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #boildata = BoilerplateData.objects.filter(boilerplate_data_id=self.kwargs['pk'])
        #data = BoilerplateGroupClassesInstances.objects.filter(classes_instances_owner= boildata.values('owner_data')[0]['owner_data'], classes_instances_group_of_boilerplate = boildata.values('group_of_boilerplate_data')[0]['group_of_boilerplate_data'])
        #g = Graph()
        #g.load(data.values('ontology_file')[0]['ontology_file'], format="turtle")
        #rdo_instance= URIRef("http://delab.csd.auth.gr/ontologies/2018/RDO-instances#" + boildata.values('title_data')[0]['title_data'])
        #g.remove((rdo_instance, None, None))
        #g.remove((None, None, rdo_instance))
        #prefix = Prefix.objects.filter(prefix_boilerplate = instance)
        #pre_instance= URIRef("http://delab.csd.auth.gr/ontologies/2018/RDO-instances#" + boildata.values('title_data')[0]['title_data'] +'_'+ prefix.values('simple_prefix')[0]['simple_prefix'])
        #g.remove((pre_instance, None, None))
        #g.remove((None, None, pre_instance))
        #main = Main.objects.filter(boilerplate_of_main = instance)
        #rdo_instance= URIRef("http://delab.csd.auth.gr/ontologies/2018/RDO-instances#" + boildata.values('title_data')[0]['title_data'] +'_'+ main.values('main_choices')[0]['main_choices'])
        #g.remove((rdo_instance, None, None))
        #g.remove((None, None, rdo_instance))
        #g.serialize(data.values('ontology_file')[0]['ontology_file'], format='turtle')
        instance.delete()

class PrefixListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = PrefixSerializer

    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        queryset_prefix = Prefix.objects.filter(prefix_boilerplate = group_pk)
        return queryset_prefix.order_by('id')


class ExtractedPropertyAPIView(APIView):
    def get(self, request, groupboil, boilerplate, format=None):
        return Response(getExtractedProperty(boilerplate))
        
class PrefixDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = PrefixSerializer
    previous_values = []
    

    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        queryset_prefix = Prefix.objects.filter(prefix_boilerplate = group_pk)
        return queryset_prefix
    
    
    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.validated_data
        if (instance['prefix']=='A' and instance['system_fun_item']=='A' and instance['state_or_verb']=='A' and instance['item_function_flow_statevalue']=='A' ):
            serializer.save(prefix='', system_fun_item='', state_or_verb='', item_function_flow_statevalue='')
            return
        #add data in thn Model BoilerplateData
        quantifier_instance,numunit_instance  = [[""], [""]]
        numinstance=''
        es_instance = instance['system_fun_item'].rsplit(':', 1)
        ev_instance =instance['state_or_verb'].rsplit(':', 1)
        ei_instance = instance['item_function_flow_statevalue'].rsplit(':', 1)      
        if instance['prefix_quantifier']:
            quantifier_instance = instance['prefix_quantifier'].rsplit(':', 1)
        if instance['prefix_mumerical_units']:
            numunit_instance = instance['prefix_mumerical_units'].rsplit(':', 1)  
        if instance['prefix_numerical']:
            numinstance= instance['prefix_numerical']
        pr = (instance['prefix'] + ' ' + es_instance[len(es_instance)-1] + ' ' + ev_instance[len(ev_instance)-1] + ' ' + quantifier_instance[len(quantifier_instance)-1] +' '+numinstance +' ' + numunit_instance[len(numunit_instance)-1]  + ' ' + ei_instance[len(ei_instance)-1])
        boildata = BoilerplateData.objects.filter(owner_data=instance['prefix_boilerplate'].owner, boilerplate_data_id = instance['prefix_boilerplate'])
        #get instances of the ontology that this boilerplate belongs
        data = BoilerplateGroupClassesInstances.objects.filter(classes_instances_owner= instance['prefix_boilerplate'].owner, classes_instances_group_of_boilerplate = boildata.values('group_of_boilerplate_data')[0]['group_of_boilerplate_data'])
        es_data = data.values('system_choices')[0]['system_choices'] + data.values('function_choices')[0]['function_choices'] + data.values('item_choices')[0]['item_choices']
        if( instance['system_fun_item'] not in es_data):
            raise APIException("PREFIX : Instance of Subject does not exist")
        ev_data = data.values('state_choices')[0]['state_choices'] + data.values('prefix_verb_choices')[0]['prefix_verb_choices']
        if( instance['state_or_verb'] not in ev_data):
            raise APIException("PREFIX : Instance of State or Verb does not exist")
        quantifier_data = data.values('quantifier_choices')[0]['quantifier_choices']
        if( instance['prefix_quantifier'] not in quantifier_data):
            raise APIException("MAIN : Instance of Quantifier does not exist")
        numunit_data = data.values('number_unit_choices')[0]['number_unit_choices']
        if( instance['prefix_mumerical_units'] not in numunit_data):
            raise APIException("MAIN : Instance of Number Unit does not exist")
        ei_data = data.values('function_choices')[0]['function_choices'] + data.values('item_choices')[0]['item_choices']+ data.values('flow_choices')[0]['flow_choices']+ data.values('state_value_choices')[0]['state_value_choices']
        if( instance['item_function_flow_statevalue'] not in ei_data):
            raise APIException("PREFIX : Instance of Object does not exist")
        #find the class of the prefix
        ontology_file = data.values('ontology_file')[0]['ontology_file']
        prefix_choices = inference.getclassofprefix(instance, ontology_file)
        if prefix_choices[0] == 'ERROR':
            raise APIException("Prefix choices not in [P1-P3]")
        else:
            if (len(self.previous_values) !=0):
                boildata.update(prefix_data = self.previous_values[0] + pr )
                self.previous_values.clear()
                serializer.save(simple_prefix = prefix_choices)
                return
            if (instance['logic_connective'] !=''):
                prefixstring = pr +' '+instance['logic_connective']+' '
                self.previous_values.append(prefixstring)
                serializer.save(simple_prefix = prefix_choices)
            else:
                boildata.update(prefix_data = pr )
                serializer.save(simple_prefix = prefix_choices)


    #Custom actions when DELETE
    def perform_destroy(self,serializer, instance):
        instance = serializer.validated_data
        instance.delete()



class BoilerplateDataListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = BoilerplateDataSerializer

    def get_queryset(self):
        group_gb = self.kwargs['groupboil']
        queryset_boil_data = BoilerplateData.objects.filter(owner_data = self.request.user, group_of_boilerplate_data = group_gb)
        return queryset_boil_data

import mimetypes
import tempfile 
import shutil

class PropertyListAPIView(APIView):
    """
    API view to retrieve property file
    """
    serializer_class = BoilerplateDataSerializer

    def get(self, request, groupboil, format=None):
        group_gb = groupboil
        queryset_boil_data = BoilerplateData.objects.filter(owner_data = self.request.user, group_of_boilerplate_data = group_gb)
        #temp_file = NamedTemporaryFile(delete=False)
        a =list(queryset_boil_data.values())
        with  tempfile.NamedTemporaryFile(mode='w+t', delete=False) as f:
            for boil_data in a:
                req= (boil_data['title_data']+': '+ boil_data['prefix_data']+','+boil_data['main_data']+','+boil_data['suffix_data'] + '.\n')
                prop =(boil_data['title_data']+'_Property: '+getExtractedProperty(boil_data['id']) + '\n \n')
                f.write(req+prop)
                f.write('-' * 150 + '\n \n')
            temp_file_name = f.name
        new_file_name = 'properties.txt'
        shutil.copyfile(temp_file_name, new_file_name)
        path = open(new_file_name, 'r')
        mime_type, _ = mimetypes.guess_type(new_file_name)
        # Set the return value of the HttpResponse
        response = HttpResponse(path, content_type=mime_type)
        # Set the HTTP header for sending to browser
        response['Content-Disposition'] = "attachment; filename=%s" % temp_file_name
        # Return the response value
        return response


class InferListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = InferSerializer

    def get_queryset(self):
        group_pk = self.kwargs['groupboil']
        queryset_infer = InferenceResults.objects.filter(infer_group_of_boilerplate = group_pk)
        return queryset_infer

class InferDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = InferSerializer

    def get_queryset(self):
        group_pk = self.kwargs['groupboil']
        queryset_infer = InferenceResults.objects.filter(infer_group_of_boilerplate = group_pk)
        return queryset_infer

    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.validated_data
        project_title = ""
        boildata= []
        prefixdata = []
        maindata = []
        suffixdata = []
        project_title =  instance['infer_group_of_boilerplate'].title_bgroup
        prefix = BoilerplateData.objects.filter(group_of_boilerplate_data = instance['infer_group_of_boilerplate'].id)
        for id in prefix:
            boildata.append(self.getboildata(id.boilerplate_data_id.id))
            prefixdata.append(self.getprefixdata(id.boilerplate_data_id.id).order_by('id'))
            maindata.append(self.getmaindata(id.boilerplate_data_id.id))
            suffixdata.append(self.getsuffixdata(id.boilerplate_data_id.id))
        #try:
        metrics, onto_file = (inference.inferencing(instance['ontology_file'], project_title, boildata, prefixdata, maindata, suffixdata))
        #except:
        #    raise APIException("Not all instances in the Ontology")
        instance.update(inference_data = metrics, ontology_file=onto_file)
        serializer.save()
    
    def getboildata(self, pr):
        return (Boilerplate.objects.filter(id = pr))
    
    def getprefixdata(self, pr):
        return (Prefix.objects.filter(prefix_boilerplate = pr))

    def getmaindata(self, pr):
        return (Main.objects.filter(boilerplate_of_main = pr))
    
    def getsuffixdata(self, pr):
        return (Suffix.objects.filter(boilerplate_of_suffix = pr))


class PrefixChoicesViewSet(ListAPIView):
    """
    API view to retrieve list of Prefix choices
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = BoilerplateGroupClassesInstancesSerializer

    def get_queryset(self):
        group_pk = self.kwargs['groupboil']
        queryset_cl= BoilerplateGroupClassesInstances.objects.filter(classes_instances_group_of_boilerplate = group_pk)
        return queryset_cl




class GetOntology(APIView):
    serializer_class = BoilerplateGroupClassesInstances

    def get(self, request, groupboil, format=None):
        data = BoilerplateGroupClassesInstances.objects.filter(classes_instances_group_of_boilerplate = groupboil)
        filename =  data.values('ontology_file')[0]['ontology_file'] + 'infer.ttl'
        filepath = data.values('ontology_file')[0]['ontology_file']+'infer.ttl'
        try :
            path = open(filepath, 'r')
        except:
            raise APIException("Fail to download ontology file")
            # Set the mime type
        mime_type, _ = mimetypes.guess_type(filepath)
        # Set the return value of the HttpResponse
        response = HttpResponse(path, content_type=mime_type)
        # Set the HTTP header for sending to browser
        response['Content-Disposition'] = "attachment; filename=%s" % filename
        # Return the response value
        return response
        
'''Return extracted Properties based on the Property patterns'''

def getExtractedProperty(group_pk):
        queryset_prefix = Prefix.objects.filter(prefix_boilerplate = group_pk).order_by('id')
        queryset_main = Main.objects.filter(boilerplate_of_main = group_pk)
        queryset_suffix = Suffix.objects.filter(boilerplate_of_suffix = group_pk)
        prefixdata= list(queryset_prefix.values())
        maindata= list(queryset_main.values())
        suffixdata= list(queryset_suffix.values())
        if(prefixdata==[] and suffixdata ==[]):
            return 'This requirement cannot export runtime monitor property.'
        mains_to_be_properties = ['M1','M2','M3','M4','M5','M6','M7','M10']
        if (maindata[0]['main_choices'] in mains_to_be_properties):
            if(maindata[0]['shall'] == 'shall'):
                mainverbsub=maindata[0]['verb'].split(':')[-1].strip() +'_'
            else:
                mainverbsub= '!'+maindata[0]['verb'].split(':')[-1].strip() + '('+ maindata[0]['sys_fun_inter'].split(':')[-1].strip()
            if(maindata[0]['statevalue_system_connection_stateset'] == '' and (maindata[0]['numerical'] == None or maindata[0]['numerical'] =='')):
                main_predicate= mainverbsub + maindata[0]['sys_fun_inter'].split(':')[-1].strip()+'('+ maindata[0]['flow_function_interface_item_system_state_stateset'].split(':')[-1].strip()+')'
            elif(maindata[0]['numerical'] == None or maindata[0]['numerical'] ==''):
                main_predicate= mainverbsub+ maindata[0]['flow_function_interface_item_system_state_stateset'].split(':')[-1].strip()+'('+maindata[0]['statevalue_system_connection_stateset'].split(':')[-1].strip() +')'
            else:
                main_predicate= mainverbsub+maindata[0]['sys_fun_inter'].split(':')[-1].strip()+'(' + maindata[0]['flow_function_interface_item_system_state_stateset'].split(':')[-1].strip()+')&('+maindata[0]['flow_function_interface_item_system_state_stateset'].split(':')[-1].strip()+' '+maindata[0]['quantifier'].split(':')[-1].strip()+' '+maindata[0]['numerical'].split(':')[-1].strip()+' '+maindata[0]['mumerical_units'].split(':')[-1].strip()+')'
            if(suffixdata !=[]):
                #update main predicate with the suffix options
                if(suffixdata[0]['s_choices'] !=''):
                    if('TimeFrameRelated' in suffixdata[0]['s_choices'] or 'TimeSpaceRelated' in suffixdata[0]['s_choices']):
                        pass
                    elif('EventRelated' in suffixdata[0]['s_choices']):
                        if( 'after' in suffixdata[0]['s_choices'] and 'Flow' in suffixdata[0]['flow']):
                            flow=suffixdata[0]['flow'].split(':')[-1].strip()
                            main_predicate = '('+main_predicate+' | !@(!'+main_predicate+' S '+flow+'))'
                        if( 'before' in suffixdata[0]['s_choices'] and 'Flow' in suffixdata[0]['flow']):
                            flow=suffixdata[0]['flow'].split(':')[-1].strip()
                            main_predicate = flow+'-> @(!'+flow+' S '+main_predicate+')'
                    elif('TimeRangeRelated' in suffixdata[0]['s_choices']):
                        num=suffixdata[0]['numerical']
                        numunti=suffixdata[0]['mumerical_units'].split(':')[-1].strip()
                        if( 'every' in suffixdata[0]['s_choices'] and 'Flow' in suffixdata[0]['flow']):
                            flow=suffixdata[0]['flow'].split(':')[-1].strip() 
                            main_predicate='(H '+flow+' & !'+main_predicate+') -> P [<='+num+' <'+numunti+'> ] '+main_predicate
                        elif('every' in suffixdata[0]['s_choices']):
                            main_predicate='!'+main_predicate+' -> P [<='+num+' <'+numunti+'> ] '+main_predicate
                        elif( 'for_a_period_of' in suffixdata[0]['s_choices'] and 'Flow' in suffixdata[0]['flow']):
                            flow=suffixdata[0]['flow'].split(':')[-1].strip()
                            main_predicate= '( !'+flow+' & '+main_predicate+') -> true S [<='+num+' <'+numunti+'> ] '+flow
                        elif( 'for_a_period_of' in suffixdata[0]['s_choices']):
                            main_predicate=main_predicate+' -> '+'true S [<='+num+' <'+numunti+'> ] FTP'
                        elif( 'within' in suffixdata[0]['s_choices'] and 'Flow' in suffixdata[0]['flow']):
                            flow=suffixdata[0]['flow'].split(':')[-1].strip()
                            main_predicate='! (!'+main_predicate+' S [<'+num+' <'+numunti+'> ]'+flow
                        elif( 'within' in suffixdata[0]['s_choices']):
                            main_predicate='H (!'+main_predicate+') -> true  S [<'+num+' <'+numunti+'> ] FTP'
                        elif( 'for_at_least' in suffixdata[0]['s_choices'] and 'Flow' in suffixdata[0]['flow']):
                            flow=suffixdata[0]['flow'].split(':')[-1].strip()
                            main_predicate=main_predicate+'S [<='+num+' <'+numunti+'> ] '+flow
                        elif( 'for_at_least' in suffixdata[0]['s_choices']):
                            main_predicate='(true S [<='+num+' <'+numunti+'> ] FTP) -> '+main_predicate
                    elif('ExecutionRelated' in suffixdata[0]['s_choices']):
                        if( 'concurrently' in suffixdata[0]['s_choices'] and 'Function' in suffixdata[0]['flow']):
                            function=suffixdata[0]['flow'].split(':')[-1].strip()
                            main_predicate=main_predicate+' | (!@'+function+')'
                        elif( 'sequentially' in suffixdata[0]['s_choices'] and 'Function' in suffixdata[0]['flow']):
                            function=suffixdata[0]['flow'].split(':')[-1].strip()
                            main_predicate=main_predicate+' | (!@'+function+' & @(!'+function+' S !'+function+'))'
            if(prefixdata!=[]):
                #add code for complex Prefix
                if(prefixdata[0]['state_or_verb'] !=''):
                    prefixverbsub=prefixdata[0]['state_or_verb'].split(':')[-1].strip()+'_'+ prefixdata[0]['system_fun_item'].split(':')[-1].strip()+'('
                    if(prefixdata[0]['prefix_numerical'] == None or prefixdata[0]['prefix_numerical'] ==''):
                        prefix1_predicate=prefixverbsub + prefixdata[0]['item_function_flow_statevalue'].split(':')[-1].strip() +')'
                    else:
                        prefix1_predicate='('+prefixverbsub + prefixdata[0]['item_function_flow_statevalue'].split(':')[-1].strip()+')&('+prefixdata[0]['item_function_flow_statevalue'].split(':')[-1].strip()+' '+prefixdata[0]['prefix_quantifier'].split(':')[-1].strip() +' '+ prefixdata[0]['prefix_numerical'].split(':')[-1].strip() +' '+ prefixdata[0]['prefix_mumerical_units'].split(':')[-1].strip() +'))'
                    prefix_predicate=prefix1_predicate
                    if(prefixdata[1]['state_or_verb'] !=''):
                        #prefix2verbsub=prefixdata[1]['state_or_verb'].split(':')[-1].strip() + '('+ prefixdata[1]['system_fun_item'].split(':')[-1].strip()
                        prefix2verbsub= prefixdata[1]['state_or_verb'].split(':')[-1].strip()+'_'+ prefixdata[1]['system_fun_item'].split(':')[-1].strip()+'('
                        if(prefixdata[1]['prefix_numerical'] == None or prefixdata[1]['prefix_numerical'] ==''):
                            prefix2_predicate=prefix2verbsub + prefixdata[1]['item_function_flow_statevalue'].split(':')[-1].strip() +')'
                        else:
                            #prefix2_predicate=prefix2verbsub + ',' + prefixdata[1]['item_function_flow_statevalue'].split(':')[-1].strip()+' '+ prefixdata[1]['prefix_quantifier'].split(':')[-1].strip() +' '+ prefixdata[1]['prefix_numerical'].split(':')[-1].strip() +' '+ prefixdata[1]['prefix_mumerical_units'].split(':')[-1].strip() +')'
                            prefix2_predicate='('+prefix2verbsub + prefixdata[1]['item_function_flow_statevalue'].split(':')[-1].strip()+')&('+prefixdata[1]['item_function_flow_statevalue'].split(':')[-1].strip()+' '+prefixdata[1]['prefix_quantifier'].split(':')[-1].strip() +' '+ prefixdata[1]['prefix_numerical'].split(':')[-1].strip() +' '+ prefixdata[1]['prefix_mumerical_units'].split(':')[-1].strip() +'))'

                        if(prefixdata[0]['logic_connective']=='OR'):
                            prefix_predicate = '('+prefix1_predicate+' | @'+prefix2_predicate+')'
                        elif(prefixdata[0]['logic_connective']=='AND'):
                            prefix_predicate = '('+prefix1_predicate+' & @'+prefix2_predicate+')'
                    if (prefixdata[0]['simple_prefix'] == 'P1'):
                        p1main= '( '+main_predicate+' | ! @(!'+main_predicate+' S '+prefix_predicate+')'+')'
                        return p1main
                    elif (prefixdata[0]['simple_prefix'] == 'P2'):
                        p2main= '('+main_predicate+' | (! @('+prefix_predicate+' & @(∃ temp, !('+prefix1_predicate+' S ('+ prefixverbsub +'temp) & !(temp='+ prefixdata[0]['item_function_flow_statevalue'].split(':')[-1].strip() +')' '))'+')))'
                        return p2main
                    elif (prefixdata[0]['simple_prefix'] == 'P3'):
                        p3main= '('+main_predicate+' | (! @('+prefix_predicate+')'
                        return p3main
            else:
                return main_predicate
        else:
            return 'This main cannot export runtime monitor property.'
